/*
 * GCLC swt, provide a swt window for console applications
 * Copyright (C) 2015-2017 E. Bigeon
 * mailto:emmanuel@bigeon.fr
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-swt:fr.bigeon.gclc.swt.HistoryTextKeyListener.java
 * Created on: Jun 9, 2016
 */
package fr.bigeon.gclc.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;

import fr.bigeon.collections.ArrayRibbon;
import fr.bigeon.collections.Ribbon;

/** A key listener to validate commands and manage the history of commands.
 *
 * @author Emmanuel Bigeon */
public final class HistoryTextKeyListener extends KeyAdapter {

    /** The size of commands history. */
    private static final int DEFAULT_HISTORY_SIZE = 10;
    /** The empty string constant. */
    private static final String EMPTY = ""; //$NON-NLS-1$
    /** The history ribbon. */
    private final Ribbon<String> commands;
    /** The current index in history search. */
    private int currentIndex = 0;
    /** The console to notify of command validation. */
    private final ConsoleDelayIO console;

    /** Create the key listener that cycle the history.
     *
     * @param console the console delayed */
    public HistoryTextKeyListener(final ConsoleDelayIO console) {
        super();
        this.console = console;
        commands = new ArrayRibbon<>(DEFAULT_HISTORY_SIZE);
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.events.KeyAdapter#keyPressed(org.eclipse.swt.events.
     * KeyEvent) */
    @Override
    public void keyPressed(final KeyEvent e) {
        pressedKeyCode(e.keyCode);
    }

    /** Indicate a pressed key combination.
     *
     * @param keyCode the pressed key code */
    public void pressedKeyCode(final int keyCode) {
        // Enter validates the command if prompting
        if (keyCode == '\r') {
            final String input = console.getInput();
            if (!input.isEmpty()) {
                commands.add(input);
            }
            console.validateInput();
            currentIndex = -1;
        }

        // Upper arrow retrieves previous commands
        if (keyCode == SWT.ARROW_UP &&
            currentIndex < commands.size() - 1) {
            currentIndex++;
            final String cmd = commands.get(commands.size() - currentIndex - 1);
            console.setInput(cmd);
        }

        // Lower arrow retrieves next commands
        if (keyCode == SWT.ARROW_DOWN) {
            if (currentIndex == 0) {
                currentIndex--;
                console.setInput(EMPTY);
            } else if (currentIndex > 0) {
                final String cmd = commands
                        .get(commands.size() - (--currentIndex) - 1);
                console.setInput(cmd);
            }
        }
    }
}